/*---------------------------------------------------------------------------
LISTING 2.   N-POINT FAST WALSH TRANSFORM IN C.

N-point fast Walsh transform.

Converted into Forth by Carey Witkov,  Dept. of Physics,
Broward Community College, 1990 May 01, from a Fortran algorithm by Tom Beer
(reference #4).

Revisions:
1990 Oct 10	Translated from Forth to C by Darwin Engwer (CIS 71121,705).
---------------------------------------------------------------------------*/

#include <stdio.h>

#define RESULTS_PER_LINE	8	/* Output format control constant. */
#define N			32	/* N = 32 for sample data. */

/* Global variables. */
int B, K, L, M;
int z [N + 1];	/* Temporary storage array. */

/* Data and output array. */
int x [N + 1] =
	{
        /*
	Sample data; 32-point sampled sine wave; scaling factor = 1000

	Note: array x begins with cell 1, not cell 0.
	      This is the opposite of the Direct Walsh Transform array x.
	*/
	0,
         195,  383,  556,  707,  831,  924,  981,  1000,
	 981,  924,  831,  707,  556,  383,  195,     0,
	-195, -383, -556, -707, -831, -924, -981, -1000,
	-981, -924, -831, -707, -556, -383, -195,     0
	};



/*---------------------------------------------------------------------------
Returns n1 to the n2 power.
---------------------------------------------------------------------------*/
static int TimesTimes (int n1, int n2)

	{
	int Result = 1;

	while (n2--)
		Result *= n1;

	return (Result);
	}



static void Initialize (void)

	{
	int i;

	for (i = 1;  i < N;  i += 2)
		{
		x [i] += x [i + 1];
		x [i + 1] = x [i] - (2 * x [i + 1]);
		}
	L = 0;
	}



/*---------------------------------------------------------------------------
Unnormalized fast walsh transform of data array x; transformed values replace
data in x.
---------------------------------------------------------------------------*/
static void FastWalshTransform (void)

	{
	int i;

	Initialize ();
	for (;;)
		{
                L++;
		M = TimesTimes (2, L);
		if (M >= N)
			break;

		B = 0;
		K = 1;
		do
			{
			for (i = 1;  i < (M + 1);  i +=2)
				{
				z [K    ] = x [B + i    ] + x [B + i + M    ];
				z [K + 1] = x [B + i    ] - x [B + i + M    ];
				z [K + 2] = x [B + i + 1] - x [B + i + M + 1];
				z [K + 3] = x [B + i + 1] + x [B + i + M + 1];
				K += 4;
				}
			B += 2 * M;
			} while (K < N);

		for (i = 1;  i < (N + 1);  i++)
			x [i] = z [i];
		}
	}



/*---------------------------------------------------------------------------
Normalizes and prints the transformed array.
---------------------------------------------------------------------------*/
static void Check (void)

	{
	int i;

	for (i = 1;  i < (N + 1);  i++)
        	printf ("%5d%c", x [i] / 32,
			(i % RESULTS_PER_LINE) ? ' ' : '\n');
	}



int main (void)

	{
	FastWalshTransform ();
	Check ();

	return (0);
	}


/* The End. */
